import polars as pl
import orjson
import typer
import os


from pathlib import Path
from iotree.core.render.funcs import basic_pbar
from typing import List, Tuple, Dict, Union, Optional, Any, Iterator, TypeVar


from ..core.isql import iSQL
from ..utils.render import console, errors
from ..core.isql_config import init_isql
from ..utils.paths import (
    base_dir, tests_dir, package_dir, pisql_home
)

isql = init_isql()

exe = typer.Typer(
    name="execute",
    help=f"""Execute a query against a Sybase ASE database . Accepts a .sql file ",
    [yellow][bold]Alias:[/] exec, e, x[/]  
    [green][bold]Example:[/]  
    >>> pisql x name.sql[/]
    """,
    rich_markup_mode="rich",
    rich_help_panel="rich",
)


@exe.callback(invoke_without_command=True, no_args_is_help=True, help="Execute a query against a Sybase ASE database . Accepts a .sql file ")
def executer(
    file: str = typer.Argument(..., help="The .sql file  to execute"),
    out: str = typer.Option(None, "-o", "--out", "--out-file", help="The output file , defaults to rich table  on console"),
    ) -> None:
    """Execute a query  against a Sybase ASE database . Accepts a .sql file \n[yellow][bold]Aliases:[/] exec, e, x[/]
    
    Args:
        file (str): The .sql file  to execute
        out (str, optional): The output file , defaults to rich table on console. Defaults to None.

        Returns:
            None: None"""
    

    file = Path(file)
    file = file.resolve()

    if not file.exists():
        console.print(errors(f"File {file} does not exist"))
        raise typer.Exit(1)
    outdir = Path(out).resolve().parent if out else pisql_home / "results"
    outfile = Path(out).resolve().name if out else file.stem
    out.parent.mkdir(parents=True, exist_ok=True)

    with basic_pbar() as pbar:
        task = pbar.add_task(f"Executing {file.name}", total=None)
        df = isql.run_sql_file(file)

        pbar.update(task, advance=50)
        pbar.update(task, description=f"Writing results to {dir}")

        if out.lower() in ["console", "rich", "stdout"]:
            console.print(f"[bold yellow]Writing raw output to stdout only.[/]")
            records = orjson.loads(df.write_json(row_oriented=True))
            console.print(tableFromRecords(records, theme="default"))
        else:
            writepath = outdir / outfile
            open(f"{writepath}.raw", "w+").write(isql.raw)
            csv_out = df.write_csv(file=f"{writepath}.csv")
            parquet_out = df.write_parquet(file=f"{writepath}.parquet",  compression="lz4")
            records = orjson.loads(df.write_json(row_oriented=True))
            console.print(tableFromRecords(records, theme="default"))

        pbar.update(task, advance=50)
        pbar.update(task, description=f"✅ Done writing results to {dir}", completed=True)
        sys.exit(0)



import typer
import os
from pathlib import Path
from typing import List, Tuple, Dict, Union, Optional, Any, Iterator

from ..core.isql_config import init_isql, iSqlCFG
from ..utils.render import console, errors

config = typer.Typer(
    name='config',
    help='Configure your regular SQL in/out folders 😎\n[bold][yellow]Aliases:[/bold] c, conf, cfg[/]',
    no_args_is_help=True,
    rich_help_panel="rich",
    rich_markup_mode='rich',
)


@config.command(name='set', help='Set a new config value for this machine')
def set_config(
    key: str = typer.Argument(..., help='The key to set the config for'),
    value: str = typer.Argument(..., help='The value to set the config for'),
):
    iSqlCFG.modifyCU(key, value)
    console.print(f'✅ [bold][green]Config set[/bold] for user [yellow]{iSqlCFG.user}:[/yellow] [red]{key}={value}[/red][/green]')

@config.command(name='get', help='Get a config value for a given user')
def get_config(
    key: str = typer.Argument(..., help='The key to get the config for'),
):
    config = iSqlCFG.getCU()
    value = config[key]
    console.print(f'✅ [bold][green]Config get[/bold] for user [yellow]{iSqlCFG.user}:[/yellow] [red]{key}={value}[/red][/green]')
    console.print(value)

@config.command(name='del', help='Delete a config value for a given user')
def del_config(
    user: str = typer.Argument(..., help='The user to delete the config for'),
    key: str = typer.Argument(..., help='The key to delete the config for'),
    ):
    iSqlCFG.delUserConfig(user, key)
    console.print(f'✅ [bold][green]Config deleted[/bold] for user [yellow]{user}:[/yellow] [red]{key}[/red][/green]')

@config.command(name='list', help='List all config values for a given user')
def list_config(
    user: str = typer.Argument(..., help='The user to list the config for'),
    ):
    config = iSqlCFG.gCU()
    console.print(f'✅ [bold][green]Config list[/bold] for user [yellow]{iSqlCFG.user}:[/yellow][/green]')
    console.print(config)

@config.command(name='reset', help='Reset the config for the current user')
def reset_config():
    iSqlCFG.reset()
    console.print(f'✅ [bold][green]Config reset[/bold] for user [yellow]{iSqlCFG.user}:[/yellow][/green]')

@config.command(name='ulist', help='List all config values for the current user')
def list_user_config():
    console.print(f'✅ [bold][green]Config list[/bold] for user [yellow]{iSqlCFG.user}:[/yellow][/green]')
    console.print(iSqlCFG.gCU())
